/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 *
 * $Id: ring.c,v 1.2.2.1 2002/10/09 19:48:33 brbarret Exp $
 *
 * Simple example of MPI showing the ring message pattern.
 */

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"


int 
main(int argc, char **argv)
{
  int j, p, tag;
  int z;
  int zout;
  int *v;
  int *xs;
  int n;
  int nv;
  int i,d;
  double start,end;

  /* Start up MPI */

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &j);
  MPI_Comm_size(MPI_COMM_WORLD, &p);
 
  /* Arbitrarily choose 201 to be our tag.  Calculate the */
  /* rank of the to process in the ring.  Use the modulus */
  /* operator so that the last process "wraps around" to rank */
  /* zero. */

  tag = 201;

  sscanf(argv[1],"%d",&n);

  if (j == 0) 
   {
    xs = (int *) malloc(n*sizeof(int));
    for (i=0;i<n;++i) scanf("%d",xs+i);
   }

  nv = n/p;
  // printf ("%d:%d:%d\n",n,p,nv);
  v = (int *) malloc(nv*sizeof(int));
  MPI_Scatter(xs,nv,MPI_INT,v,nv,MPI_INT,0,MPI_COMM_WORLD);

  start = MPI_Wtime();

  z = 0;
  for (i=0;i<nv;++i) z+=v[i];
  end = MPI_Wtime();

  MPI_Reduce(&z,&zout,1,MPI_INT,MPI_SUM,0,MPI_COMM_WORLD);
  
  // notice zout valid only in process 0
  // printf("%d\n",zout);
  if (0 == j)
   {
    printf("sum is %d in %fs\n",zout,end-start);
   }
  MPI_Finalize();
  return 0;
}
